﻿Public Class SortedArrayList(Of T As IComparable)
    Inherits ArrayList(Of T)

    Public Sub BubbleSort()
        Dim swapped = True
        Dim pass = 0
        ' bereits sortierte Einträge am Ende des Arrays
        While swapped
            swapped = False

            For i = 0 To Count - 1 - pass - 1
                If Me(i).CompareTo(Me(i + 1)) > 0 Then
                    Swap(i, i + 1)
                    swapped = True
                End If
            Next
            pass += 1
        End While
    End Sub

    Public Sub InsertionSort()
        For i = 1 To Count - 1
            For j = i To 1 Step -1
                If Me(j).CompareTo(Me(j - 1)) < 0 Then
                    Swap(j, j - 1)
                Else
                    Exit For
                End If
            Next
        Next
    End Sub

    Public Sub BucketSort()
        Dim max = Integer.Parse(Me.Max.ToString())
        Dim min = Integer.Parse(Me.Min.ToString())
        Dim n = max - min + 1

        ' Partitionen anlegen
        Dim partitions = New SortedArrayList(Of T)(n - 1) {}
        For i = 0 To n - 1
            partitions(i) = New SortedArrayList(Of T)()
        Next

        ' Einträge auf Partitionen verteilen
        For Each item As T In Me
            Dim pn As Integer = CInt(Convert.ChangeType(item, GetType(Integer))) - min
            partitions(pn).Add(item)
        Next

        ' Partitionen konkatenieren
        n = 0
        For Each pn As SortedArrayList(Of T) In partitions
            For Each item As T In pn
                Me(n) = item
                n += 1
            Next
        Next
    End Sub

    Public ReadOnly Property Max() As T
        Get
            If Count = 0 Then
                Throw New ArgumentException("No items in list")
            End If

            Dim itemMax As T = Me(0)
            For Each item As T In Me
                If item.CompareTo(itemMax) > 0 Then
                    itemMax = item
                End If
            Next

            Return itemMax
        End Get
    End Property

    Public ReadOnly Property Min() As T
        Get
            If Count = 0 Then
                Throw New ArgumentException("No items in list")
            End If

            Dim itemMin As T = Me(0)
            For Each item As T In Me
                If item.CompareTo(itemMin) < 0 Then
                    itemMin = item
                End If
            Next

            Return itemMin
        End Get
    End Property

    Public Sub QuickSort()
        QSort(0, Count - 1)
    End Sub

    Private Sub QSort(lower As Integer, upper As Integer)
        If lower >= upper Then
            Return
        End If

        Dim pivot = (lower + upper) / 2 ' alternativ: Random().Next(0, Count)
        Dim pivotNew = Divide(lower, upper, pivot)

        QSort(lower, pivotNew - 1)
        QSort(pivotNew + 1, upper)
    End Sub

    Private Function Divide(lower As Integer, upper As Integer, pivot As Integer) As Integer
        Dim pivotValue = Me(pivot)
        Dim pivotNew = lower

        ' Pivotelement nach oben verschieben
        Swap(pivot, upper)

        For i = lower To upper - 1
            If Me(i).CompareTo(pivotValue) < 0 Then ' kleiner
                Swap(i, pivotNew)
                pivotNew += 1
            End If
        Next

        ' Pivotwert an den neuen einsortierten Pivotindex setzen
        Swap(upper, pivotNew)

        Return pivotNew
    End Function

    Public Sub MergeSort()
        ' items auf protected setzen, damit Zugriff
        MSort(items, Count)
    End Sub

    Private Shared Sub Merge(target As SortedArrayList(Of T), left As SortedArrayList(Of T), right As SortedArrayList(Of T))
        Dim l = 0, r = 0

        Dim count = left.Count + right.Count

        While count > 0
            If l >= left.Count Then
                ' der letzte linke wurde bereits verglichen
                target.Add(right(r))
                r += 1
            ElseIf r >= right.Count Then
                ' der letzte rechte wurde bereits verglichen
                target.Add(left(l))
                l += 1
            ElseIf left(l).CompareTo(right(r)) < 0 Then
                target.Add(left(l))
                l += 1
            Else
                target.Add(right(r))
                r += 1
            End If

            count -= 1
        End While
    End Sub


    Private Sub MSort(items As T(), n As Integer)
        If n = 1 Then ' Aufteilung bis auf 1 Element
            Return
        End If

        Dim mid As Integer = n / 2
        Dim leftSize = mid
        Dim rightSize = n - leftSize

        Dim left As T() = New T(leftSize - 1) {}
        Dim right As T() = New T(rightSize - 1) {}

        Array.Copy(items, 0, left, 0, leftSize)
        Array.Copy(items, mid, right, 0, rightSize)

        MSort(left, left.Length)
        MSort(right, right.Length)

        Merge(items, left, right)
    End Sub

    Private Shared Sub Merge(target As T(), left As T(), right As T())
        Dim l = 0, r = 0, t = 0

        Dim count = left.Length + right.Length

        While count > 0
            If l >= left.Length Then
                ' der letzte linke wurde bereits verglichen
                target(t) = right(r)
                r += 1
            ElseIf r >= right.Length Then
                ' der letzte rechte wurde bereits verglichen
                target(t) = left(l)
                l += 1
            ElseIf left(l).CompareTo(right(r)) < 0 Then
                target(t) = left(l)
                l += 1
            Else
                target(t) = right(r)
                r += 1
            End If

            t += 1
            count -= 1
        End While
    End Sub

    Public Sub MergeSortParallel()
        MSortParallel(items, Count)
    End Sub

    Private Sub MSortParallel(items As T(), n As Integer)
        If n = 1 Then
            ' Aufteilung bis auf 1 Element
            Return
        End If

        Dim mid As Integer = n / 2
        Dim leftSize = mid
        Dim rightSize = n - leftSize

        Dim left As T() = New T(leftSize - 1) {}
        Dim right As T() = New T(rightSize - 1) {}

        Array.Copy(items, 0, left, 0, leftSize)
        Array.Copy(items, mid, right, 0, rightSize)

        Task.WaitAll(New Task() {Task.Run(Sub() MSortParallel(left, left.Length)), Task.Run(Sub() MSortParallel(right, right.Length))})

        Merge(items, left, right)
    End Sub

    Public Shared Function Merge(lists As SortedArrayList(Of T)()) As SortedArrayList(Of T)
        ' Count berücksichtigen
        If lists.Length < 2 Then
            Return Nothing
        End If

        Dim sortedArray = New SortedArrayList(Of T)()

        For i = 0 To lists.Length - 2
            SortedArrayList(Of T).Merge(sortedArray, lists(i), lists(i + 1))
        Next

        Return sortedArray
    End Function

    Private Sub Swap(index1 As Integer, index2 As Integer)
        Dim item As T = Me(index1)

        Me(index1) = Me(index2)
        Me(index2) = item
    End Sub

    Private Shared Sub Swap(ByRef item1 As T, ByRef item2 As T)
        Dim item As T = item1

        item1 = item2
        item2 = item
    End Sub

    Public Function BinarySearch(item As T) As Integer
        Dim lower = 0
        Dim upper = Count - 1

        While lower <= upper
            Dim mid As Integer = lower + (upper - lower) / 2
            ' alternativ: >> 1 anstatt / 2
            Dim o = item.CompareTo(Me(mid))

            If o = 0 Then
                Return mid
            ElseIf o > 0 Then
                lower = mid + 1
            Else
                ' oben weitersuchen
                upper = mid - 1
                ' unten weitersuchen
            End If
        End While
        Return Not lower
    End Function

    Public Function BinarySearchRecursive(item As T) As Integer
        Return Search(item, 0, Count - 1)
    End Function

    Private Function Search(item As T, lower As Integer, upper As Integer) As Integer
        If lower > upper Then
            Return Not lower
        End If

        Dim mid As Integer = lower + (upper - lower) / 2

        Dim o = item.CompareTo(Me(mid))

        If o = 0 Then
            Return mid
        ElseIf o > 0 Then
            Return Search(item, mid + 1, upper)
        Else
            ' oben weitersuchen
            Return Search(item, lower, mid - 1)
        End If
        ' unten weitersuchen
    End Function


    Public Overrides Function ToString() As String
        Dim s As String = ""

        For i = 0 To Count - 1
            s += Me(i).ToString() + ";"
        Next

        Return s
    End Function

End Class

