﻿Imports System.IO
Imports System.Text
Imports Newtonsoft.Json

Public Class Disc

    Sub New(discId As String, discTitle As String, year As Integer, genre As String)
        Me.DiscId = discId
        Me.DiscTitle = discTitle
        Me.Year = year
        Me.Genre = genre
    End Sub

    Public Property DiscId As String
    Public Property DiscTitle As String
    Public Property Year As Integer
    Public Property Genre As String

    Public Overrides Function ToString() As String
        Return $"{DiscId} | {DiscTitle} | {Year} | {Genre}"
    End Function

End Class

Public Class Discs

    <JsonProperty>
    Private discTitles As RedBlackTree(Of String, Disc)

    <JsonProperty>
    Private musicFolder As String

    Sub New(musicFolder As String)
        Me.musicFolder = musicFolder
        Me.discTitles = New RedBlackTree(Of String, Disc)()
    End Sub
    Sub Load()
        For Each file In Directory.EnumerateFiles(musicFolder, "*.", SearchOption.AllDirectories)
            ReadDiscFile(file)
        Next
    End Sub

    Private Sub ReadDiscFile(file As String)
        Using reader = New StreamReader(file, Encoding.Default)

            Dim discTitle As String = Nothing, genre = Nothing, discId = Nothing
            Dim year = 0

            Dim line As String = reader.ReadLine()
            While line IsNot Nothing
                If (Not line(0).Equals("#"c)) Then

                    Dim index = line.IndexOf("="c)
                    Dim key = line.Substring(0, index)
                    Dim value = line.Substring(index + 1)

                    If (key = "DISCID") Then
                        discId = value
                    ElseIf (key = "DTITLE") Then
                        discTitle = value
                    ElseIf (key = "DGENRE") Then
                        genre = value
                    ElseIf (key = "DYEAR") Then
                        Dim converted = Integer.TryParse(value, year)
                    End If
                End If
                line = reader.ReadLine()
            End While

            If (discId <> Nothing) Then
                discTitles.Add(discId, New Disc(discId, discTitle, year, genre))
            End If
        End Using
    End Sub
    Function Read(discId As String) As Disc
        If (discTitles.Contains(discId) = True) Then
            Return discTitles(discId)
        End If
        Return Nothing
    End Function

End Class
