Module Program
    Sub Main(args As String())

        Dim count As Integer = 0
        Dim argsCorrect As Boolean = If(args.Length > 0, Integer.TryParse(args(0), count), False)

        If Not argsCorrect Then
            Console.WriteLine("Aufruf: ProfileMethods <AnzahlDurchlufe>" & vbLf & "Beispiel: ProfileMethods 10000")
            Console.WriteLine("<AnzahlDurchlufe> auf 10.000 gesetzt!")
            count = 10000
        End If

        Dim profiler = New Profiler(count)

        Dim arrayList = New ArrayList(Of Integer)()
        Dim linkedList = New LinkedList(Of Integer)()
        Dim arrayListSystem = New System.Collections.Generic.List(Of Integer)()
        Dim linkedListSystem = New System.Collections.Generic.LinkedList(Of Integer)()

        profiler.AddMethod(arrayList, "Add", True)
        profiler.AddMethod(arrayList, "Remove", True)
        profiler.AddMethod(linkedList, "Add", True)
        profiler.AddMethod(linkedList, "Remove", True)
        profiler.AddMethod(arrayListSystem, "Add", True)
        profiler.AddMethod(arrayListSystem, "Remove", True)
        profiler.AddMethod(linkedListSystem, "AddLast", True)
        profiler.AddMethod(linkedListSystem, "RemoveLast", True)
        profiler.AddMethod(arrayList, "Add", False)
        profiler.AddMethod(arrayList, "RemoveAt", True)
        profiler.AddMethod(arrayListSystem, "Add", False)
        profiler.AddMethod(arrayListSystem, "RemoveAt", True)

        'profiler.AddMethod(arrayList,        "Add",     false);
        'profiler.AddMethod(arrayList,        "IndexOf", true);
        'profiler.AddMethod(arrayListSystem,  "Add",     false);
        'profiler.AddMethod(arrayListSystem,  "IndexOf", true);
        'profiler.AddMethod(linkedList,       "Add",     false);
        'profiler.AddMethod(linkedList,       "Contains", true);
        'profiler.AddMethod(linkedListSystem, "AddLast",  false);
        'profiler.AddMethod(linkedListSystem, "Contains", true);

        profiler.Run()

    End Sub


    Friend Class Profiler
        Private count As Integer
        Private countData As Integer ' Anzahl Messwerte

        Private watch As New Stopwatch()

        Private list As New ArrayList(Of Method)()

        Private Class Method
            Public Property Obj() As Object
            Public Property Name() As String
            Public Property Protocol() As Boolean
        End Class

        Public Sub New(count As Integer)
            Me.count = count
            countData = count / 1000

            Console.WriteLine("Methode;Anzahl;Zeit(Sek.);Zeit(Ticks);Zeit pro Element(ms)")
        End Sub

        Public Sub AddMethod(obj As Object, name As String, protocol As Boolean)
            list.Add(New Method() With {
                    .Obj = obj,
                    .Name = name,
                    .Protocol = protocol
                })
        End Sub

        Public Sub Run()
            For i As Integer = 0 To list.Count - 1
                Dim p = list(i)
                Dim obj = p.Obj

                Dim type = obj.GetType()
                Dim methods = type.GetMethods()
                Dim method As Reflection.MethodInfo = Nothing

                For Each methodInfo As Reflection.MethodInfo In methods
                    If methodInfo.Name.Equals(p.Name) Then
                        method = methodInfo
                        Exit For
                    End If
                Next

                If method Is Nothing Then
                    Console.WriteLine(type.ToString() + "." + p.Name + " existiert nicht!")
                    Continue For
                End If

                Dim countParams = method.GetParameters().Length

                watch.Restart()

                For n As Integer = count - 1 To 0 Step -1
                    Show(type.Name + "." + method.Name, n, p.Protocol)
                    method.Invoke(obj, If(countParams = 0, Nothing, New Object() {n}))
                Next

                watch.Stop()
            Next
        End Sub

        Private Sub Show(method As String, n As Integer, protocol As Boolean)
            If (count - n) Mod countData = 0 AndAlso protocol = True Then
                ' alle countData Zeitausgabe
                Dim info As String
                info = method + ";" +
                    (count - n).ToString() + ";" +
                    (watch.ElapsedMilliseconds / 1000.0).ToString() + ";" +
                    watch.ElapsedTicks.ToString() + ";" +
                    (CDbl(watch.ElapsedMilliseconds) / (count - n)).ToString()

                Console.WriteLine(info)
            End If
        End Sub
    End Class
End Module
